@@ -1,4 +1,6 @@
 import asyncio
+import struct
+import warnings
 from functools import cached_property
 from typing import Callable, List, Optional
 
@@ -11,7 +13,7 @@ from . import (
     ReadingEnumFailed,
     utils,
 )
-from .constants import WIZARD_SPEED
+from .constants import WIZARD_SPEED, ntdll
 from .memory import (
     CurrentActorBody,
     CurrentClientObject,
@@ -22,6 +24,8 @@ from .memory import (
     DuelPhase,
     HookHandler,
     CurrentRenderContext,
+    TeleportHelper,
+    MovementTeleportHook,
 )
 from .mouse_handler import MouseHandler
 from .utils import (
@@ -30,6 +34,7 @@ from .utils import (
     get_window_title,
     set_window_title,
     get_window_rectangle,
+    wait_for_value,
 )
 
 
@@ -59,12 +64,17 @@ class Client:
         self.root_window = CurrentRootWindow(self.hook_handler)
         self.render_context = CurrentRenderContext(self.hook_handler)
 
+        self._teleport_helper = TeleportHelper(self.hook_handler)
+
         self._template_ids = None
         self._is_loading_addr = None
         self._world_view_window = None
-        self._is_infinite_patched = False
-        # (addr, bytes)
-        self._infinite_patch_og_info = None
+
+        # (pos_pointer_addr, hash_addr)
+        self._position_holder_globals = None
+
+        # for teleport
+        self._je_instruction_forward_backwards = None
 
     def __repr__(self):
         return f"<Client {self.window_handle=} {self.process_id=}>"
@@ -188,7 +198,7 @@ class Client:
         return self._world_view_window
 
     async def activate_hooks(
-        self, *, wait_for_ready: bool = True, timeout: float = None
+            self, *, wait_for_ready: bool = True, timeout: float = None
     ):
         """
         Activate all memory hooks but mouseless
@@ -298,7 +308,7 @@ class Client:
         return int(used), int(total)
 
     async def wait_for_zone_change(
-        self, name: Optional[str] = None, *, sleep_time: Optional[float] = 0.5
+            self, name: Optional[str] = None, *, sleep_time: Optional[float] = 0.5
     ):
         """
         Wait for the client's zone to change
@@ -381,7 +391,15 @@ class Client:
         await self.body.write_yaw(yaw)
         await utils.timed_send_key(self.window_handle, Keycode.W, move_seconds)
 
-    async def teleport(self, xyz: XYZ, yaw: float = None, *, move_after: bool = True):
+    # TODO: 2.0 remove move_after as it isn't needed anymore
+    async def teleport(
+            self,
+            xyz: XYZ,
+            yaw: float = None,
+            *,
+            move_after: bool = True,
+            wait_on_inuse: bool = False,
+    ):
         """
         Teleport the client
 
@@ -390,21 +408,32 @@ class Client:
             yaw: yaw to set or None to not change
 
         Keyword Args:
-            move_after: If the client should rotate some to update the player model position
+            move_after: depreciated
+            wait_on_inuse: If we should wait for the update bool to be False
         """
-        await self.patch_infinite_loading()
-        await self.body.write_position(xyz)
+        # we do this because the old teleport only required the body hook
+        client_object = await self.body.parent_client_object()
+        client_object_addr = await client_object.read_base_address()
+
+        await self._teleport_object(client_object_addr, xyz, wait_on_inuse)
 
         if move_after:
+            warnings.warn(DeprecationWarning("Move after will be removed in 2.0"))
             await self.send_key(Keycode.D, 0.1)
 
         if yaw is not None:
             await self.body.write_yaw(yaw)
 
-    # TODO: test what the speed loss is using this instead of normal .teleport
-    async def pet_teleport(self, xyz: XYZ, yaw: float = None, *, move_after: bool = True):
+    async def pet_teleport(
+            self,
+            xyz: XYZ,
+            yaw: float = None,
+            *,
+            move_after: bool = True,
+            wait_on_inuse: bool = False,
+    ):
         """
-        Teleport the client's pet
+        Teleport while playing as pet
 
         Args:
             xyz: xyz to teleport to
@@ -413,34 +442,100 @@ class Client:
         Keyword Args:
             move_after: If the client's pet should rotate some to update the player model position
         """
-        # when you are playing as pet .client_object is the pet ClientObject
-        pet_actor_body = await self.client_object.actor_body()
+        client_object_addr = await self.client_object.read_base_address()
 
-        await self.patch_infinite_loading()
-        await pet_actor_body.write_position(xyz)
+        await self._teleport_object(client_object_addr, xyz, wait_on_inuse)
 
         if move_after:
+            warnings.warn(DeprecationWarning("Move after will be removed in 2.0"))
             await self.send_key(Keycode.D, 0.1)
 
         if yaw is not None:
-            await pet_actor_body.write_yaw(yaw)
+            await self.body.write_yaw(yaw)
 
-    async def patch_infinite_loading(self):
-        """
-        Patches infinite loading
-        """
-        if self._is_infinite_patched:
-            return
+    async def _teleport_object(self, object_address: int, xyz: XYZ, wait_on_inuse: bool = False):
+        if await self._teleport_helper.should_update():
+            if not wait_on_inuse:
+                raise ValueError("Tried to teleport while should update bool is set")
 
-        cmp_addr = await self.hook_handler.pattern_scan(
-            b"\x80\x3D.....\x0F\x85...........\xE8....\x39"
+            await wait_for_value(self._teleport_helper.should_update, True)
+
+        jes = await self._get_je_instruction_forward_backwards()
+
+        if not self.hook_handler._check_if_hook_active(MovementTeleportHook):
+            raise RuntimeError("MOVEMENT TELEPORT NOT ACTIVE")
+
+        await self._teleport_helper.write_target_object_address(object_address)
+        await self._teleport_helper.write_position(xyz)
+        await self._teleport_helper.write_should_update(True)
+
+        for je in jes:
+            await self.hook_handler.write_bytes(je, b"\x90" * 6)
+
+    async def _get_je_instruction_forward_backwards(self):
+        """
+        this method returns the two je instruction addresses :)
+        """
+        if self._je_instruction_forward_backwards is not None:
+            return self._je_instruction_forward_backwards
+
+        movement_state_instruction_addr = await self.hook_handler.pattern_scan(
+            rb"\x8B\x5F\x70\xF3",
+            module="WizardGraphicalClient.exe"
         )
 
-        # +2 offset [4 bytes long]
-        ac_flag_offset = await self.hook_handler.read_typed(cmp_addr + 2, "int")
+        self._je_instruction_forward_backwards = (
+            movement_state_instruction_addr + 15,
+            movement_state_instruction_addr + 24,
+        )
 
-        ac_flag_addr = cmp_addr + 7 + ac_flag_offset
+        return self._je_instruction_forward_backwards
 
-        await self.hook_handler.write_typed(ac_flag_addr, True, "bool")
+    async def _patch_position_holder(self, xyz: XYZ):
+        """
+        Patches position holder to a new xyz
+        """
+        position_pointer_addr, position_hash_addr = await self._get_position_holder_globals()
 
-        self._is_infinite_patched = True
+        packed_position_bytes = struct.pack("<fff", *xyz)
+        position_hash = self._calculate_position_hash(packed_position_bytes)
+
+        position_addr = await self.hook_handler.read_typed(position_pointer_addr, "unsigned long long")
+        await self.hook_handler.write_bytes(position_addr, packed_position_bytes)
+        await self.hook_handler.write_typed(position_hash_addr, position_hash, "unsigned int")
+
+    @staticmethod
+    def _calculate_position_hash(xyz_bytes: bytes):
+        res = 0x811c9dc5
+
+        for byte in xyz_bytes:
+            res = (byte ^ res) * 0x1000193
+
+        return res & 0xffffffff
+
+    # TODO: remove
+    async def _get_position_holder_globals(self):
+        """
+        returns: positional(pointer), hash
+        """
+        if self._position_holder_globals:
+            return self._position_holder_globals
+
+        player_position_func_pattern = rb"\x48\x83\xEC\x28\x48\x8D.....\x48\x8D.....\xE8....\x33\xC0\x48\x8D....." \
+                                       rb"\x48\x89.....\x48\x89.....\x88.....\x89.....\xE8....\x89.....\x48\x83\xC4" \
+                                       rb"\x28\xC3"
+
+        player_position_func_addr = await self.hook_handler.pattern_scan(
+            player_position_func_pattern,
+            module="WizardGraphicalClient.exe"
+        )
+
+        position_pointer_offset_mov_addr = player_position_func_addr + 32
+        position_pointer_offset_rip = await self.hook_handler.read_typed(
+            position_pointer_offset_mov_addr + 3,
+            "unsigned int"
+        )
+        position_pointer = position_pointer_offset_mov_addr + 7 + position_pointer_offset_rip
+
+        self._position_holder_globals = (position_pointer, position_pointer + 12)
+        return self._position_holder_globals
